# This file is part of FatxFast.
#
#    FatxFast is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    FatxFast is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with FatxFast.  If not, see <http://www.gnu.org/licenses/>.

# Versioning based on: 
# http://en.wikipedia.org/wiki/Versioning#Designating_development_stage
__author__ = "dryatu (c) 2013"
__version__ = "1.2.3"


from FatxFast import camera
from FatxFast.objects.buffs import BUFF_CLASSES
from FatxFast.objects.player import Player
from FatxFast.objects.ghost import Ghost
from FatxFast import events
from FatxFast.gui.boxwidget import BoxWidget
from FatxFast.input import KeyListener, MouseListener
import pygame

class GameMode(KeyListener, MouseListener):
    
    def __init__(self, scene, **kwargs):

        if scene.time is None:
            scene.time = 0
        self.scene = scene 
        self.camera = camera.Camera()
        self.scene.levelsystem.load_level()
        self.create_players()    
        self.hotkeys = scene.manager.game.config.hotkeys
        self.scene.paused = False
        self.level_reload = False
        if not self.scene.loaded_from_save:
            self.load_level_assets()

    def _create_player(self, player_name, camera, add_to_world=True, 
        type=Player):
        x = self.scene.tilemap.properties.get("start_x", 64)
        y = self.scene.tilemap.properties.get("start_y", 64)
        setattr(self, player_name, self.scene.create_character(type, 
            pos=(x, y), add_to_world=add_to_world))
        player = getattr(self, player_name)
        player.name = player_name
        self._load_playerdata_from_save(player)
        getattr(camera, "follow_target")(player)
        self.scene.add_player(player,camera) 

    def create_players(self):
        self._create_player("player1", self.camera)

    def end(self, player, text=""):
        self.scene.game.config.games_played += 1
        self.scene.remove_players()
        self.scene.tilemap.unload_map()
        self.scene.playing = False
    
        self.scene.manager.clear()
        self.scene.manager.push_scene_by_name("introscene")
        self.scene.manager.push_scene_by_name("gameendscene", text=text,
            played_mode=self.__class__.__name__.lower())

    def key_pressed(self, key, utf, mod):
        """if key == self.hotkeys["solve"]:
            self.scene.solve()"""
        if key == self.hotkeys["player1-map"]:
            self.open_map(self.player1)
        elif key == self.hotkeys["player1-walk"]:
            self.player1.walk = True
        elif key == pygame.K_o:
            self._create_player("ghost", self.camera, add_to_world=True,
                type=Ghost)

    def key_released(self, key):
        if key == self.hotkeys["player1-walk"]:
            self.player1.walk = False

    def keys_held(self):
        keys = super(GameMode, self).keys_held()
        player1 = self.player1
        if keys[self.hotkeys["player1-up"]]:
            player1.set_accel(y=-player1.accelspeed)
        elif keys[self.hotkeys["player1-down"]]:
            player1.set_accel(y=player1.accelspeed)
        else:
            player1.set_accel(y=0)
        if keys[self.hotkeys["player1-left"]]:
            player1.set_accel(x=-player1.accelspeed)
        elif keys[self.hotkeys["player1-right"]]:
            player1.set_accel(x=player1.accelspeed)
        else:
            player1.set_accel(x=0)
        return keys

    def level_up(self, player): 
        self.scene.save.playerdata[player.name]["points"] = player.points
        self.scene.remove_players()
        self.scene.tilemap.unload_map()
        self.level_reload = True

    def load_level_assets(self):
        if "maps_left" in self.scene.levelsystem.level_complexity["data"]:
            self.player1.maps_left += \
                self.scene.levelsystem.level_complexity["data"]["maps_left"]

    def _load_playerdata_from_save(self, player):
        if player.name in self.scene.save.playerdata:
            player_config = self.scene.save.playerdata[player.name]
            if "points" in player_config:
                player.points = player_config["points"]
            if "pos" in player_config:
                player.rect.x, player.rect.y = player_config["pos"]
                del player_config["pos"]
            if "buffs" in player_config:
                for buff in player_config["buffs"].values():
                    player.buffs.add(
                        (BUFF_CLASSES[buff[0]](player, buff[1])))
                del player_config["buffs"]
            if "maps_left" in player_config:
                player.maps_left = player_config["maps_left"]
                del player_config["maps_left"]
        else:
            self.scene.save.playerdata[player.name] = {}

    def open_map(self, player):
        if player.maps_left:
            player.maps_left -= 1
            self.scene.open_map()

    def pause(self):
        self.scene.game.input.remove_key_listener(self)
        self.scene.game.input.remove_mouse_listener(self)
   
    def resume(self):
        self.scene.hud.remove_players() 
        self.scene.hud.add_player(self.player1, self.camera)
        self.scene.game.input.add_key_listener(self)
        self.scene.game.input.add_mouse_listener(self)

    def start(self):
        self.scene.game.input.add_key_listener(self)
        self.scene.game.input.add_mouse_listener(self)
    
    def update(self, dt):
        self.keys_held()
        self.scene.tilemap.update(dt)
        self.camera.update()
        self.scene.levelsystem.leveltime += dt
